package eeds;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import xmlgp.XMLGP;
import xmlgp.Experiment;
import xmlgp.Generation;
import xmlgp.Individual;
import xmlgp.Node;
import xmlgp.Project;
import xmlgp.Representation;
import xmlgp.Treatment;
import xmlgp.Trial;

import static xmlgp.RepresentationType.TREE;


/**
 * This class standardize GP results.
 * @author dharani
 *
 */
public class StandardizeOutput 
{
	private String inputFileName; //post fix input file name
	private String outputXmlFileName; //standardized output XML file name
	

	// constructor for serializing the standardized output to file
	StandardizeOutput(String inputFileName, String outputXmlFileName) 
	{
		this.inputFileName = inputFileName;
		this.outputXmlFileName = outputXmlFileName;
	}
	
	// constructor for serializing the standardized output to console
	StandardizeOutput(String inputFileName) 
	{
		this.inputFileName = inputFileName;		
	}

	/**
	 * Forms XML standardization
	 */
	private Project formXML() 
	{
		Node exprNode = null;
		Project project = null;
		
		try 
		{
			FileInputStream finstream = new FileInputStream(inputFileName);
			// Get the object of DataInputStream
			DataInputStream in = new DataInputStream(finstream);
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String strLine;
			
			Representation representation = null;
			List<Representation> representations = null;
			Individual individual = new Individual();
			List<Individual> individuals = new ArrayList<Individual>();
			
			// Read File Line By Line
			while ((strLine = br.readLine()) != null) 
			{				
				exprNode = Util.constructNode(strLine);
				representation = new Representation();
				representations = new ArrayList<Representation>();
				representation.setAttributes(1, TREE, null, exprNode);				
				representations.add(representation);

				individual = new Individual();				
				individual.setAttributes(1, null, representations);
				individuals.add(individual);				
			}

			Generation generation = new Generation();
			generation.setAttributes(1, null, individuals);			
			List<Generation> generations = new ArrayList<Generation>();
			generations.add(generation);

			Trial trial = new Trial();
			trial.setGenerations(generations);
			trial.setId(1);
			List<Trial> trials = new ArrayList<Trial>();
			trials.add(trial);

			Treatment treatment = new Treatment();
			treatment.setTrials(trials);
			treatment.setId(1);
			List<Treatment> treatments = new ArrayList<Treatment>();
			treatments.add(treatment);

			Experiment experiment = new Experiment();
			experiment.setTreatments(treatments);
			experiment.setId(1);
			List<Experiment> experiments = new ArrayList<Experiment>();
			experiments.add(experiment);

			project = new Project();
			project.setExperiments(experiments);
			project.setId(1);
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		} 
		
		return project;
	}	
	
	/**
	 * Serializes the output to file
	 */
	private void serializeToFile()
	{
		Project project = formXML();		
		XMLGP.serializeToFile(project, outputXmlFileName);		
	}
	
	/**
	 * Serializes the output to console
	 */
	private void serializeToStream()
	{
		Project project = formXML();
		XMLGP.serializeToStream(project);
	}

	// main method
	public static void main(String[] args) 
	{		
		StandardizeOutput stdOutput = null;
		int argLen = args.length;
			
		//depending on arguments length serialize to file or console 
		//arguments are input postfix file name, output xml file name
		if (argLen == 2) 
		{
			stdOutput = new StandardizeOutput(args[0], args[1]);
			stdOutput.serializeToFile();
		}
		//argument is input postfix
		else if(argLen == 1)
		{
			stdOutput = new StandardizeOutput(args[0]);
			stdOutput.serializeToStream();
		}			
	}
}