package eeds;

import java.util.ArrayList;
import java.util.Stack;
import java.util.StringTokenizer;

import xmlgp.Node;
import static xmlgp.NodeType.NON_TERMINAL;
import static xmlgp.NodeType.TERMINAL;

/**
 * This class has the utility methods for the EEDS framework.
 * @author dharani
 *
 */
public class Util {
	
	/**
	 * Checks the token if it is an operator.
	 * @param token the operator
	 * @return true/false
	 */
    public static boolean isOperator(String token) 
	{
		return (token.equals("+") || token.equals("-") || token.equals("*") || token.equals("/") || token.equals("S"));
	}
 
    /**
     * Converts postfix expression string to node.
     * @param expr the expression string to be converted into node
     * @return the constructed node
     */
    public static Node constructNode(String expr) 
	{
		Stack<Node> stack = new Stack<Node>();
		
		Node cNode = null;
		String token;
		StringTokenizer tokenizer = new StringTokenizer(expr);
		Node node = null;		

		//get each token in the string and form the expression node
		while (tokenizer.hasMoreTokens()) 
		{
			token = tokenizer.nextToken();
			
			if(isOperator(token)) 
			{
				if(!stack.empty())
				{	
					ArrayList<Node> children = new ArrayList<Node>();
					cNode = stack.pop();
					children.add(cNode);
					
					if(!"S".equals(token))
					{
						cNode = stack.pop();
						children.add(cNode);
					}					
					
					// create non-terminal node
					node = new Node();
					//node.setId(id);
					node.setType(NON_TERMINAL);
					node.setValue(token);									
					node.setChildren(children);
					stack.push(node);
				}
			} 
			else 
			{
				// create terminal node
				node = new Node();
				//node.setId(id);
				node.setType(TERMINAL);
				node.setValue(token);				
				stack.push(node);
			}		
		}
		stack.clear();
		return node;
	}      
}